@@ -1,5 +1,3 @@
-
-
 class FeatureLibError(Exception):
     def __init__(self, message, location):
         Exception.__init__(self, message)
@@ -8,8 +6,7 @@ class FeatureLibError(Exception):
     def __str__(self):
         message = Exception.__str__(self)
         if self.location:
-            path, line, column = self.location
-            return f"{path}:{line}:{column}: {message}"
+            return f"{self.location}: {message}"
         else:
             return message
 
@@ -22,5 +19,4 @@ class IncludedFeaNotFound(FeatureLibError):
             "The following feature file should be included but cannot be found: "
             f"{Exception.__str__(self)}"
         )
-        path, line, column = self.location
-        return f"{path}:{line}:{column}: {message}"
+        return f"{self.location}: {message}"
