@@ -1,7 +1,10 @@
-from fontTools.misc.py23 import *
+from fontTools.colorLib.builder import buildCOLR
 from fontTools.ttLib import TTFont, newTable
-from fontTools.varLib import build, load_designspace
+from fontTools.ttLib.tables import otTables as ot
+from fontTools.varLib import build, load_designspace, _add_COLR
 from fontTools.varLib.errors import VarLibValidationError
+import fontTools.varLib.errors as varLibErrors
+from fontTools.varLib.models import VariationModel
 from fontTools.varLib.mutator import instantiateVariableFont
 from fontTools.varLib import main as varLib_main, load_masters
 from fontTools.varLib import set_default_weight_width_slant
@@ -10,6 +13,8 @@ from fontTools.designspaceLib import (
 )
 from fontTools.feaLib.builder import addOpenTypeFeaturesFromString
 import difflib
+from copy import deepcopy
+from io import BytesIO
 import os
 import shutil
 import sys
@@ -83,12 +88,11 @@ class BuildTest(unittest.TestCase):
         lines = []
         with open(path, "r", encoding="utf-8") as ttx:
             for line in ttx.readlines():
-                # Elide ttFont attributes because ttLibVersion may change,
-                # and use os-native line separators so we can run difflib.
+                # Elide ttFont attributes because ttLibVersion may change.
                 if line.startswith("<ttFont "):
-                    lines.append("<ttFont>" + os.linesep)
+                    lines.append("<ttFont>\n")
                 else:
-                    lines.append(line.rstrip() + os.linesep)
+                    lines.append(line.rstrip() + "\n")
         return lines
 
     def expect_ttx(self, font, expected_ttx, tables):
@@ -813,6 +817,72 @@ class BuildTest(unittest.TestCase):
 
         assert ds_loaded.instances[0].location == {"weight": 0, "width": 50}
 
+    def test_varlib_build_incompatible_features(self):
+        with pytest.raises(
+            varLibErrors.ShouldBeConstant,
+            match = """
+
+Couldn't merge the fonts, because some values were different, but should have
+been the same. This happened while performing the following operation:
+GPOS.table.FeatureList.FeatureCount
+
+The problem is likely to be in Simple Two Axis Bold:
+Expected to see .FeatureCount==2, instead saw 1
+
+Incompatible features between masters.
+Expected: kern, mark.
+Got: kern.
+"""):
+
+            self._run_varlib_build_test(
+                designspace_name="IncompatibleFeatures",
+                font_name="IncompatibleFeatures",
+                tables=["GPOS"],
+                expected_ttx_name="IncompatibleFeatures",
+                save_before_dump=True,
+            )
+
+    def test_varlib_build_incompatible_lookup_types(self):
+        with pytest.raises(
+            varLibErrors.MismatchedTypes,
+            match = r"'MarkBasePos', instead saw 'PairPos'"
+        ):
+            self._run_varlib_build_test(
+                designspace_name="IncompatibleLookupTypes",
+                font_name="IncompatibleLookupTypes",
+                tables=["GPOS"],
+                expected_ttx_name="IncompatibleLookupTypes",
+                save_before_dump=True,
+            )
+
+    def test_varlib_build_incompatible_arrays(self):
+        with pytest.raises(
+            varLibErrors.ShouldBeConstant,
+            match = """
+
+Couldn't merge the fonts, because some values were different, but should have
+been the same. This happened while performing the following operation:
+GPOS.table.ScriptList.ScriptCount
+
+The problem is likely to be in Simple Two Axis Bold:
+Expected to see .ScriptCount==1, instead saw 0"""
+        ):
+            self._run_varlib_build_test(
+                designspace_name="IncompatibleArrays",
+                font_name="IncompatibleArrays",
+                tables=["GPOS"],
+                expected_ttx_name="IncompatibleArrays",
+                save_before_dump=True,
+            )
+
+    def test_varlib_build_variable_colr(self):
+        self._run_varlib_build_test(
+            designspace_name='TestVariableCOLR',
+            font_name='TestVariableCOLR',
+            tables=["GlyphOrder", "fvar", "glyf", "COLR", "CPAL"],
+            expected_ttx_name='TestVariableCOLR-VF',
+            save_before_dump=True,
+        )
 
 def test_load_masters_layerName_without_required_font():
     ds = DesignSpaceDocument()
@@ -919,5 +989,38 @@ class SetDefaultWeightWidthSlantTest(object):
         assert ttFont["post"].italicAngle == -12.0
 
 
+def test_variable_COLR_without_VarIndexMap():
+    # test we don't add a no-op VarIndexMap to variable COLR when not needed
+    # https://github.com/fonttools/fonttools/issues/2800
+
+    font1 = TTFont()
+    font1.setGlyphOrder([".notdef", "A"])
+    font1["COLR"] = buildCOLR({"A": (ot.PaintFormat.PaintSolid, 0, 1.0)})
+    # font2 == font1 except for PaintSolid.Alpha
+    font2 = deepcopy(font1)
+    font2["COLR"].table.BaseGlyphList.BaseGlyphPaintRecord[0].Paint.Alpha = 0.0
+    master_fonts = [font1, font2]
+
+    varfont = deepcopy(font1)
+    axis_order = ["XXXX"]
+    model = VariationModel([{}, {"XXXX": 1.0}], axis_order)
+
+    _add_COLR(varfont, model, master_fonts, axis_order)
+
+    colr = varfont["COLR"].table
+
+    assert len(colr.BaseGlyphList.BaseGlyphPaintRecord) == 1
+    baserec = colr.BaseGlyphList.BaseGlyphPaintRecord[0]
+    assert baserec.Paint.Format == ot.PaintFormat.PaintVarSolid
+    assert baserec.Paint.VarIndexBase == 0
+
+    assert colr.VarStore is not None
+    assert len(colr.VarStore.VarData) == 1
+    assert len(colr.VarStore.VarData[0].Item) == 1
+    assert colr.VarStore.VarData[0].Item[0] == [-16384]
+
+    assert colr.VarIndexMap is None
+
+
 if __name__ == "__main__":
     sys.exit(unittest.main())
